==> gordo_components-0.42.0-py3-none-any.whl: gordo_components/builder/local_build.py <==
# -*- coding: utf-8 -*-

import io
from typing import Iterable, Tuple, Union

from sklearn.base import BaseEstimator

from gordo_components.workflow.config_elements.normalized_config import NormalizedConfig
from gordo_components.workflow.workflow_generator.workflow_generator import (
    get_dict_from_yaml,
)
from gordo_components.builder import ModelBuilder


def local_build(config_str: str) -> Iterable[Tuple[Union[BaseEstimator, None], dict]]:
    """
    Build model(s) from a bare Gordo config file locally.

    This is very similar to the same steps as the normal workflow generation and subsequent
    Gordo deployment process makes. Should help developing locally,
    as well as giving a good indication that your config is valid for deployment
    with Gordo.

    Parameters
    ----------
    config_str: str
        The raw yaml config file in string format.

    Examples
    --------
    >>> import numpy as np
    >>> config = '''
    ... machines:
    ...       - dataset:
    ...           tags:
    ...             - SOME-TAG1
    ...             - SOME-TAG2
    ...           target_tag_list:
    ...             - SOME-TAG3
    ...             - SOME-TAG4
    ...           train_end_date: '2019-03-01T00:00:00+00:00'
    ...           train_start_date: '2019-01-01T00:00:00+00:00'
    ...           asset: asgb
    ...           data_provider:
    ...             type: RandomDataProvider
    ...         metadata:
    ...           information: Some sweet information about the model
    ...         model:
    ...           gordo_components.model.anomaly.diff.DiffBasedAnomalyDetector:
    ...             base_estimator:
    ...               sklearn.pipeline.Pipeline:
    ...                 steps:
    ...                 - sklearn.decomposition.pca.PCA
    ...                 - sklearn.multioutput.MultiOutputRegressor:
    ...                     estimator: sklearn.linear_model.base.LinearRegression
    ...         name: crazy-sweet-name
    ... '''
    >>> models_n_metadata = local_build(config)
    >>> assert len(list(models_n_metadata)) == 1

    Returns
    -------
    Iterable[Tuple[Union[BaseEstimator, None], dict]]
        A generator yielding tuples of models and their metadata.
    """
    config = get_dict_from_yaml(io.StringIO(config_str))
    normed = NormalizedConfig(config, project_name="local-build")
    for machine in normed.machines:
        yield ModelBuilder(
            name=machine.name,
            model_config=machine.model,
            data_config=machine.dataset.to_dict(),
            metadata=machine.metadata,
            evaluation_config=machine.evaluation,
        ).build()

==> gordo_components-0.44.0-py3-none-any.whl: gordo_components/builder/local_build.py <==
# -*- coding: utf-8 -*-

import io
from typing import Iterable, Tuple, Union

from sklearn.base import BaseEstimator

from gordo_components.workflow.config_elements.normalized_config import NormalizedConfig
from gordo_components.workflow.workflow_generator.workflow_generator import (
    get_dict_from_yaml,
)
from gordo_components.builder import ModelBuilder
from gordo_components.builder.mlflow_utils import mlflow_context, log_metadata


def local_build(
    config_str: str, enable_mlflow: bool = True, workspace_kwargs: dict = {}
) -> Iterable[Tuple[Union[BaseEstimator, None], dict]]:
    """
    Build model(s) from a bare Gordo config file locally.

    This is very similar to the same steps as the normal workflow generation and subsequent
    Gordo deployment process makes. Should help developing locally,
    as well as giving a good indication that your config is valid for deployment
    with Gordo.

    Parameters
    ----------
    config_str: str
        The raw yaml config file in string format.
    enable_mlflow: bool
        Flag to enable Mlflow logging of model building results. With `enable_mlflow`
        set to `True`, passing `workspace_kwargs` an empty dict (default) will result in
        local MLflow logging, while passing a dict of keyword arguments as defined in
        :func:`~gordo_components.builder.mlflow_utils.get_mlflow_client` will result in
        remote logging.
    workspace_kwargs: dict
        AzureML Workspace configuration to use for remote MLFlow tracking. See
        :func:`gordo_components.builder.azure_utils.get_mlflow_client`.

    Examples
    --------
    >>> import numpy as np
    >>> config = '''
    ... machines:
    ...       - dataset:
    ...           tags:
    ...             - SOME-TAG1
    ...             - SOME-TAG2
    ...           target_tag_list:
    ...             - SOME-TAG3
    ...             - SOME-TAG4
    ...           train_end_date: '2019-03-01T00:00:00+00:00'
    ...           train_start_date: '2019-01-01T00:00:00+00:00'
    ...           asset: asgb
    ...           data_provider:
    ...             type: RandomDataProvider
    ...         metadata:
    ...           information: Some sweet information about the model
    ...         model:
    ...           gordo_components.model.anomaly.diff.DiffBasedAnomalyDetector:
    ...             base_estimator:
    ...               sklearn.pipeline.Pipeline:
    ...                 steps:
    ...                 - sklearn.decomposition.pca.PCA
    ...                 - sklearn.multioutput.MultiOutputRegressor:
    ...                     estimator: sklearn.linear_model.base.LinearRegression
    ...         name: crazy-sweet-name
    ... '''
    >>> models_n_metadata = local_build(config, enable_mlflow=False)
    >>> assert len(list(models_n_metadata)) == 1

    Returns
    -------
    Iterable[Tuple[Union[BaseEstimator, None], dict]]
        A generator yielding tuples of models and their metadata.
    """
    config = get_dict_from_yaml(io.StringIO(config_str))
    normed = NormalizedConfig(config, project_name="local-build")
    for machine in normed.machines:
        model, metadata = ModelBuilder(
            name=machine.name,
            model_config=machine.model,
            data_config=machine.dataset.to_dict(),
            metadata=machine.metadata,
            evaluation_config=machine.evaluation,
        ).build()

        if enable_mlflow:
            # This will enforce a single interactive login and automatically
            # generate a new key for each model built
            with mlflow_context(
                name=machine.name, workspace_kwargs=workspace_kwargs
            ) as (mlflow_client, run_id):
                log_metadata(mlflow_client, run_id, metadata)

        yield model, metadata
